(*
 *  bserv, Internet Relay Chat services, bservunit.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  some modifications by peter green
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
unit bservunit;

interface


uses
  SysUtils, Classes,
  blinklist,bstuff,btime,pgdebugout,{pgmail,}
  {$ifndef FPC}
    messages,
  {$endif}
  {$ifdef win32}
    Windows, {Messages,}winsock,wsocket;
  {$else}
    lsocket,linux,sockets,unixsignals;
  {$endif}
type
  tsc=class
    procedure connected(sender:tobject; error:word);
    procedure closed(sender:tobject; error:word);
    procedure receive(sender:tobject; error:word);

  end;

var
  sc:tsc;
  reconnectcountdown:integer;
  wanttoconnect:boolean;
  socketbusy:boolean;
  {$ifdef win32}
    sock:twsocket;
  {$else}
    sock:tlsocket;
  {$endif}
  shutdownreason:string;

procedure startconnect;
procedure globalshutdown;
procedure startup;
procedure timehandler;

procedure processmessages;
procedure initapplication;
procedure runapplication;
procedure cleanupapplication;

procedure filelog(filename,line:string);

implementation

uses
{$ifdef win32}
  bwinnt,
{$endif}

bsignal,bparse,bconfig,bservicemngr,bserver,buser,bconsts,bcmds;

var
  recvq:string;

{$ifdef win32}
function  MyWindowProc(ahWnd   : HWND;
                       auMsg   : Integer;
                       awParam : WPARAM;
                       alParam : LPARAM): Integer; stdcall; forward;


var
  hwndmain:hwnd;
  MyWindowClass : TWndClass = (style         : 0;
                                 lpfnWndProc   : @MyWindowProc;
                                 cbClsExtra    : 0;
                                 cbWndExtra    : 0;
                                 hInstance     : 0;
                                 hIcon         : 0;
                                 hCursor       : 0;
                                 hbrBackground : 0;
                                 lpszMenuName  : nil;
                                 lpszClassName : 'MyWindowClass');

function ProcessMessage : Boolean;
var
    Msg : TMsg;
begin
    Result := FALSE;
    if PeekMessage(Msg, 0, 0, 0, PM_REMOVE) then begin
      Result := TRUE;
      DispatchMessage(Msg);
    end;
end;
{$endif}
procedure processmessages;
begin
  // in the linux version this is a stub as the linux version does 
  // not use messages 
  {$ifdef win32}
    while processmessage do;
  {$endif}
end;

{$ifdef win32}

function MyWindowProc(
    ahWnd   : HWND;
    auMsg   : Integer;
    awParam : WPARAM;
    alParam : LPARAM): Integer; stdcall;
var
    MsgRec : TMessage;
begin
    Result := 0;  // This means we handled the message

        {MsgRec.hwnd    := ahWnd;}
        MsgRec.wParam  := awParam;
        MsgRec.lParam  := alParam;

        case auMsg of
        WM_TIMER: begin
          if awparam = 123 then Timehandler;
        end;
        WM_CLOSE: begin
          {WriteLn('Closing');}
          DestroyWindow(ahWnd);
        end;
        WM_DESTROY: begin
          Cleanupapplication;
        end;


        {       WM_KEYUP:   writeln('WM_KEYUP'); }
{       WM_CHAR:    writeln('WM_CHAR');  }
        else
            Result := DefWindowProc(ahWnd, auMsg, awParam, alParam)
        end;

end;

{$endif}

procedure serverdown;
var
  a:integer;

begin
  if me = nil then exit;

  {shutdown bots}
  for a := 0 to maxserviceunit do if serviceunitarray[a] <> nil then
  if @serviceunitarray[a].onshutdown <> nil then
  serviceunitarray[a].onshutdown;

  {close signal port}
  signalstop;
  

  if isconnected then begin
    send(me.idstr+' '+TOK_SQUIT+' '+me.name+' 0 :'+shutdownreason);
    sock.close;
  end;

  {shutdown core}
  writecfg;
  me.destroy;
  me := nil;

  sock.destroy;
  sock := nil;
end;

procedure globalshutdown;
begin
  {$ifdef win32}
    reportstopping;
  {$endif}
  serverdown;
  {$ifdef win32}

    reportstopped;
  {$endif}

end;

procedure startconnect;
begin
  if isconnected then exit;
  if socketbusy then exit;
  wanttoconnect := true;
  socketbusy := true;

  sock.addr := opt.connectaddr;
  sock.port := opt.connectport;
  sock.proto := 'tcp';
  sock.connect;
end;

procedure tsc.connected;
begin
  debugout('sc.connected called '+inttostr(error));
  if error = 0 then begin

    socketbusy := false;


{    setsockopt(sock.Hsocket,IPPROTO_TCP,TCP_NODELAY,@a,4);}
    recvq := '';
    isconnected := true;

    onconnected;

  end else begin
    isconnected := false;
  end;
end;


procedure Tsc.receive(sender : tobject; error : word);
var
  a,b,c,d:integer;
  s:string;
  goed:boolean;
begin


  recvq := recvq + sock.receivestr;

  goed := false;
  a := 1;
  b := length(recvq);
  d := 0;
  repeat
    if length(recvq)=0 then exit; // a squit has resulted in the connection
                                  // being broken and reset the recvq
    c := a;

    while (a <= b) and (recvq[a] <> #13) and (recvq[a] <> #10) do inc(a);

    if a <= b then begin
      s := copy(recvq,c,a-c);
      d := a;
      if s <> '' then parsecommand(false,s);

    end else goed := true;

    inc(a);
    //writeln('goed');
  until goed;
  if d > 0 then delete(recvq,1,d);
  sendflush;
end;




procedure tsc.closed;
begin
  //writeln(isconnected);
  //readln;
  {$ifdef win32}
    if isconnected then sock.close;
  {$endif}
  socketbusy := false;
  isconnected := false;
  online := false;
  if hostservuser <> nil then hostservuser.destroy;
  hostservuser := nil;

end;

procedure startup;
var
  a:integer;
begin
  {$ifdef win32}  
    sock := twsocket.create(nil);
    // sock.multithreaded := true;
    // the emial unit used by the A module uses a seperate thread to send the
    // email hence the need for this
  {$else}
    sock := tlsocket.create;
    sock.ontimer := timehandler;
  {$endif}
  sock.onsessionconnected := sc.connected;
  sock.ondataavailable := sc.receive;
  sock.onsessionclosed := sc.closed;

  {startup core}
  me := adduser;
  setnickname(me,opt.myservername);
  me.fullname := opt.info;
  addserver(me,opt.myservernum,maxbot,nil);
  mynum := p10inttostr(opt.myservernum,SSlen);

  {open signal port}
  signalstart;


  {
  on startup stuff.
  call onstartup of any service units which i want to have running now
  }
  for a := maxserviceunit downto 0 do begin
    if serviceunitarray[a] <> nil then
    if @serviceunitarray[a].onstartup <> nil then
    serviceunitarray[a].onstartup;
  end;
end;

procedure timehandler;
var
  a:integer;
  s:tserviceunit;
begin
  debugout('timehandler called');
  //debugout(inttostr(ord(signalsource.connected)));
  if reconnectcountdown > 0 then dec(reconnectcountdown);
  if reconnectcountdown <= 0 then begin
    if wanttoconnect then startconnect;
    reconnectcountdown := 4;
  end;

  {call timehandler of a few other units which are not bots}
  btime.timehandler;
  {$ifndef win32}
    unixsignals.timehandler;
  {$endif}

  //pgmail.timehandler;
  {call timehandler of bots}

  for a := maxserviceunit downto 0 do begin
    s := serviceunitarray[a];
    if s <> nil then if @s.ontimer <> nil then s.ontimer;
  end;
  sendflush;
end;

{
start up bots button

  for a := 0 to maxserviceunit do if serviceunitarray[a] <> nil then
  if @serviceunitarray[a].onstartup <> nil then
  serviceunitarray[a].onstartup;

shutdown bots button

  for a := 0 to maxserviceunit do if serviceunitarray[a] <> nil then
  if @serviceunitarray[a].onshutdown <> nil then
  serviceunitarray[a].onshutdown;

want connect button

  startconnect;

want disconnect button

  wanttoconnect := false;
  sock.close;

on terminate, call shutdown

}

procedure initapplication;
begin
  {$ifdef win32}
    if Windows.RegisterClass(MyWindowClass) = 0 then Exit;


    hWndMain := CreateWindowEx(WS_EX_TOOLWINDOW,
                                 MyWindowClass.lpszClassName,
                                 '',        { Window name   }
                                 WS_POPUP,  { Window Style  }
                                 0, 0,      { X, Y          }
                                 0, 0,      { Width, Height }
                                 0,         { hWndParent    }
                                 0,         { hMenu         }
                                 HInstance, { hInstance     }
                                 nil);      { CreateParam   }
    if hWndMain = 0 then halt;
    settimer(hwndmain,123,1000,nil);
  {$endif}

  startup;
  startconnect;

  //pgmail.init;
end;


procedure runapplication;
{$ifdef win32}
  var
    MsgRec : TMsg;
{$endif}
begin
  {$ifdef win32}
    while GetMessage(MsgRec, 0, 0, 0) do begin
        {TranslateMessage(MsgRec);}
        debugout(inttostr(msgrec.message));
        debugout('dispatchmessade called');
        DispatchMessage(MsgRec)
    end;
  {$else}
    messageloop

  {$endif}
end;

procedure cleanupapplication;
begin
  globalshutdown;
  {$ifdef win32}
    if hWndMain <> 0 then begin
        DestroyWindow(hWndMain);
        hWndMain := 0;
    end;  
  {$endif}
end;


procedure filelog(filename,line:string);
var
  t:textfile;
begin
  assignfile(t,filename);
  {$i-}append(t);{$i+}
  if ioresult <> 0 then {$i-}rewrite(t);{$i+}
  if ioresult <> 0 then exit;
  writeln(t,'['+inttostr(unixtime)+' '+timestrshort(unixtime)+'] '+line);
  closefile(t);
end;

end.
