(*
 *  bserv, Internet Relay Chat services, bworldexception.pas
 *  Copyright (C) 2002 Bas Steendijk
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *)
unit bworldexception;

{unit for maintaining clone limit exception list}

interface

uses bchannel;

var
  exceptionlist:tban;

procedure loadexception(const filename:string);
procedure saveexception(const filename:string);

implementation

uses bconfig,blinklist,sysutils;

procedure saveexception(const filename:string);
var
  t:textfile;
  b:tban;

begin
  assignfile(t,filename);
  filemode := 2;
  {$i-}rewrite(t);{$i+}
  if ioresult <> 0 then begin
    {saving failed}
    exit;
  end;
  b := tban(exceptionlist);
  while b <> nil do begin
    writeln(t,'[exception]');
    writeln(t,'mask=',b.mask);
    writeln(t,'sender=',b.sender);
    writeln(t,'limit=',b.date);
    writeln(t);
    b := tban(b.next);
  end;
  closefile(t);
end;


procedure clearexception;
var
  b,b2:tban;
begin
  b := tban(exceptionlist);
  while b <> nil do begin
    b2 := tban(b.next);
    linklistdel(tlinklist(exceptionlist),tlinklist(b));
    b.destroy;
    b := b2;
  end;
end;

var
  currentexception:tban;

procedure loadexceptionfinish;
begin
  if currentexception <> nil then begin
    if currentexception.mask <> '' then begin
      linklistadd(tlinklist(exceptionlist),tlinklist(currentexception));
    end else currentexception.destroy;
  end;
end;

procedure loadexceptionproc(sv,sn:string);
begin
  if sv = '[EXCEPTION]' then begin
    loadexceptionfinish;
    currentexception := tban.create;
  end;
  if currentexception = nil then exit;
  if sv = 'MASK' then currentexception.mask := sn;
  if sv = 'LIMIT' then currentexception.date := strtointdef(sn,0);
  if sv = 'SENDER' then currentexception.sender := sn;
end;

procedure loadexception(const filename:string);
begin
  clearexception;
  currentexception := nil;
  readcfg(filename,@loadexceptionproc);
  loadexceptionfinish;
  currentexception := nil;
end;

end.
